// 0.3.0之前版本引用方式（1个文件）
// import "../../miniprogram_npm/tencentcloud-webar/lib.js";
// 0.3.0及之后版本引用方式（2个文件 + 按需初始化3d模块）
import '../../miniprogram_npm/tencentcloud-webar/lib.js';
import '../../miniprogram_npm/tencentcloud-webar/core.js';
// 按需初始化3d插件，如果不需要3d则可以不引用
import '../../miniprogram_npm/tencentcloud-webar/lib-3d.js';
import { plugin3d } from '../../miniprogram_npm/tencentcloud-webar/plugin-3d'
// 导入 ArSdk
import { ArSdk } from "../../miniprogram_npm/tencentcloud-webar/index.js";

import sha256 from '../../utils/sha256';

/** ----- 鉴权配置 ----- */

/**
 * 腾讯云账号 APPID
 * 
 * 进入[腾讯云账号中心](https://console.cloud.tencent.com/developer) 即可查看 APPID
 */
const APPID = '';

/**
 * Web LicenseKey
 * 
 * 登录音视频终端 SDK 控制台的[Web License 管理](https://console.cloud.tencent.com/vcube/web)，创建项目即可获得 LicenseKey
 */
const LICENSE_KEY = '';

/**
 * 计算签名用的密钥 Token
 * 
 * 注意：此处仅用于 DEMO 调试，正式环境中请将 Token 保管在服务端，签名方法迁移到服务端实现，通过接口提供，前端调用拉取签名，参考
 * [签名方法](https://https://cloud.tencent.com/document/product/616/71370#.E7.AD.BE.E5.90.8D.E6.96.B9.E6.B3.95)
 */
const token = '';

Page({
  data: {
    makeupList: [],
    stickerList: [],
    filterList: [],
    recording: false,
    videoSaving: false
  },
  getCanvasNode(id) {
    return new Promise(resolve => {
      this.createSelectorQuery()
        .select(`#${id}`)
        .node()
        .exec(res => {
          const canvasNode = res[0].node;
          resolve(canvasNode);
        });
    });
  },
  /**
   * 定义获取签名方法
   * 注意：签名方法推荐在服务端实现，通过接口提供，前端调用拉取签名，如：
   * getSignature(){
   *   return new Promise((resolve,reject)=>{
   *     wx.request({
   *       url: 'https://xxx.com/get-ar-sign',
   *       method: 'GET',
   *       success(res) {
   *         console.log('getSignature ok', res)
   *          resolve(res.data);
   *       },
   *       fail(e){
   *         console.log('getSignature error', e)
   *       }
   *     })
   *   })
   * },
   * 此处为了帮助您快速跑通所以在前端计算签名
   */
  getSignature() {
    const timestamp = Math.round(new Date().getTime() / 1000);
    const signature = sha256(timestamp + token + APPID + timestamp).toUpperCase();
    return { signature, timestamp };
  },
  async start() {
    const outputCanvas = await this.getCanvasNode('main-canvas');
    const auth = {
      licenseKey: LICENSE_KEY,
      appId: APPID,
      authFunc: this.getSignature
    };
    const config = {
      auth,
      camera: {
        width: 720,
        height: 1280,
      },
      output: outputCanvas,
      loading: {
        enable: false
      },
      // 初始美颜效果（可选）
      beautify: {
        whiten: 0.1
      }
    };
    const ar = new ArSdk(config);
    console.log('new sdk', ar);
    // created回调里可以开始获取内置特效与滤镜列表
    ar.on('created', async () => {
      console.log('sdk created');
      // 获取内置美妆、贴纸
      ar.getEffectList({
        Type: 'Preset',
      }).then((res) => {
        const list = res.map(item => ({
          name: item.Name,
          id: item.EffectId,
          cover: item.CoverUrl,
          url: item.Url,
          label: item.Label,
          type: item.PresetType,
        }));
        console.log('effectList', list);
        this.setData({
          makeupList: list.filter(item=>item.label.indexOf('美妆')>=0),
          stickerList: list.filter(item=>item.label.indexOf('贴纸')>=0)
        });
      }).catch((e) => {
        console.error(e);
      });
      // 内置滤镜
      ar.getCommonFilter().then((res) => {
        const list = res.map(item => ({
          name: item.Name,
          id: item.EffectId,
          cover: item.CoverUrl,
          url: item.Url,
          label: item.Label,
          type: item.PresetType,
        }));
        console.log('filterList', list);
        this.setData({
          filterList: list
        });
      }).catch((e) => {
        console.error(e);
      });
    });
    ar.on('ready', async () => {
      // 在 ready 回调里及该事件之后，可使用三种方法来设置美颜特效：setBeautify/setEffect/setFilter
      console.log('sdk ready');
      // 通过setBeautify设置美颜效果
      ar.setBeautify({
        whiten: 0.4, // 美白 0-1
        dermabrasion: 0.5, // 磨皮 0-1
        lift: 0.3, // 瘦脸 0-1
        shave: 0, // 削脸 0-1
        eye: 0, // 大眼 0-1
        chin: 0, // 下巴 0-1
      });
      // 通过created回调中创建的美妆、贴纸列表交互设置效果(setEffect的输入参数支持三种格式，详见SDK接入指南)
      ar.setEffect([{
        id: this.data.makeupList[1].id,
        filterIntensity: 0 // 将美妆内置的滤镜强度设为0，仅显示后面setFilter的滤镜
      }, this.data.stickerList[10].id]);
      // 通过created回调中创建的滤镜列表交互设置滤镜效果(setFilter第二个参数1表示强度，详见SDK接入指南）
      ar.setFilter(this.data.filterList[0].id, 1);
    });
    ar.on('error', error => {
      console.error('sdk error', error);
    });
    this.ar = ar;
  },
  // 拍照
  async takePhoto() {
    const {uint8ArrayData, width, height} = this.ar.takePhoto(); // takePhoto 方法返回当前画面的 buffer 数据
    const photoCanvasNode = await this.getCanvasNode('photo-canvas');
    photoCanvasNode.width = parseInt(width);
    photoCanvasNode.height = parseInt(height);
    const ctx = photoCanvasNode.getContext('2d');
    // 用 sdk 返回的数据创建 ImageData 对象
    const imageData = photoCanvasNode.createImageData(uint8ArrayData, width, height);
    // 将 ImageData 对象绘制到 canvas 上
    ctx.putImageData(imageData,0,0,0,0,width,height);
    // 将 canvas 保存为本地图片
    wx.canvasToTempFilePath({
      canvas: photoCanvasNode,
      x: 0,
      y: 0,
      width: width,
      height: height,
      destWidth: width,
      destHeight: height,
      success: (res) => {
        console.log('拍照成功',res);
        wx.saveImageToPhotosAlbum({
          filePath: res.tempFilePath,
          success: () => {
            console.log('保存照片成功')
          }
        });
      }
    })
  },
  // 开始录像
  async startRecord() {
    if (this.data.recording) return;
    await this.ar.startRecord();
    this.setData({
      recording: true
    });
  },
  // 结束录像
  async stopRecord() {
    if (!this.data.recording) return;
    this.setData({
      videoSaving:true
    });
    const res = await this.ar.stopRecord();
    console.log('录像数据',res);
    wx.saveVideoToPhotosAlbum({
      filePath: res.tempFilePath,
      success: () => {
        console.log('保存录像成功')
      }
    });
    this.setData({
      videoSaving:false,
      recording: false
    })
  },
  onClickRecord() {
    if (this.data.recording) {
      this.stopRecord()
    } else {
      this.startRecord()
    }
  },
  onReady() {
    console.log('onReady');
    !this.ar && this.start();
  },
  async onHide() {
    console.log('onhide');
    if (this.ar) {
      await this.stopRecord();
      this.ar.stop()
    };
  },
  onShow: function () {
    console.log('onShow');
    this.ar && this.ar.start();
  },
  async onUnload() {
    console.log('onUnload');
    this.ar.destroy();
    this.ar = null;
  }
});
